#ifndef VERSION_H
#define VERSION_H

#define VER_FILEVERSION             1,0,0,0
#define VER_FILEVERSION_STR         "1.0.0.0\0"

#define VER_PRODUCTVERSION          1,0,0,0
#define VER_PRODUCTVERSION_STR      "1.0\0"

#define VER_COMPANYNAME_STR         "Olive Team"
#define VER_FILEDESCRIPTION_STR     "Olive"
#define VER_INTERNALNAME_STR        "Olive"
#define VER_LEGALCOPYRIGHT_STR      "Copyright © 2018 Olive Team"
#define VER_LEGALTRADEMARKS1_STR    "All Rights Reserved"
#define VER_LEGALTRADEMARKS2_STR    VER_LEGALTRADEMARKS1_STR
#define VER_ORIGINALFILENAME_STR    "Olive.exe"
#define VER_PRODUCTNAME_STR         "Olive"

#define VER_COMPANYDOMAIN_STR       "www.olivevideoeditor.org"

#endif // VERSION_H
