/***

  Olive - Non-Linear Video Editor
  Copyright (C) 2019 Olive Team

  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>.

***/

/* Hack that forces checked QPushButtons to use dark color */


/* Node styling */
NodeViewItemWidget {
  qproperty-titlebarColor: #a0a0ff;
  qproperty-borderColor: #000000;
}

/* Timeline playhead styling */
TimelinePlayhead {
  qproperty-playheadColor: #ff0000;
  qproperty-playheadHighlightColor: rgba(0, 0, 0, 0.25);
}
